@@ -1,9 +1,8 @@
 /*
  * Toonloop
  *
- * Copyright 2010 Alexandre Quessy
- * <alexandre@quessy.net>
- * http://www.toonloop.com
+ * Copyright (c) 2010 Alexandre Quessy <alexandre@quessy.net>
+ * Copyright (c) 2010 Tristan Matthews <le.businessman@gmail.com>
  *
  * Toonloop is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,12 +24,18 @@
 #include "moviesaver.h"
 #include "saverworker.h"
 #include "subprocess.h" // TODO: use glib instead of subprocess.h
-#include "timing.h"
-//#include "timing.h" // get_iso_dateime_for_now
 
 SaverWorker::SaverWorker(MovieSaver *owner) :
     owner_(owner)
 {
+    datetime_started_ = "";
+    final_file_name_ = "";
+}
+
+void SaverWorker::set_final_options(const std::string &datetime_started, const std::string &final_file_name)
+{
+    datetime_started_ = datetime_started;
+    final_file_name_ = final_file_name;
 }
 
 /**
@@ -50,8 +55,7 @@ void SaverWorker::operator()()
     //std::cout << "In the saving thread for clip #" << clip_id << " with " << num_images << " images" << std::endl; // TODO
     
     // TODO: create symlinks
-    std::string datetime_started = timing::get_iso_datetime_for_now();
-    fs::path directory = fs::path(g_get_tmp_dir()) / fs::path("toonloop-" + datetime_started);
+    fs::path directory = fs::path(g_get_tmp_dir()) / fs::path("toonloop-" + datetime_started_);
     //std::cout << "----------------------------------" << std::endl;
     //std::cout << "tmp dir: " << directory.string() << std::endl;
     //std::cout << "----------------------------------" << std::endl;
@@ -77,7 +81,8 @@ void SaverWorker::operator()()
         // Will containt the image_paths, file_extension and format, plus the path to the image directory, etc.
         snprintf(buffer, BUFSIZE, "%06d.jpg", i);
         fs::path to_p = fs::path(owner_->current_task_.image_paths_[i]);
-        fs::path from_p = directory / fs::path(buffer);
+        fs::path from_p = fs::path(directory.string());
+        from_p /= std::string(buffer);
         //std::cout << " $ ln -s " << to_p.string() << " " << from_p.string() << std::endl;
 
         if (!fs::exists(to_p)) 
@@ -106,19 +111,23 @@ void SaverWorker::operator()()
     // TODO: Make width/height configurable
     // TODO: support AVI file.. 
     //  mencoder "mf://*.jpg" -mf fps=5 -o test.avi -ovc lavc -lavcopts vcodec=msmpeg4v2:vbitrate=800
-    std::string command = std::string("mencoder mf://") + directory.string() + std::string("/*.jpg -quiet -mf w=640:h=480:fps=" + fps + ":type=jpg -ovc lavc -lavcopts vcodec=mjpeg -oac copy -of lavf -lavfopts format=mov -o ") + output_movie.string(); 
-    std::cout << "Lauching $ " << command << std::endl;  
-    bool ret_val = subprocess::run_command(command); // blocking call
+    
+    int clip_w = owner_->current_task_.width_;
+    int clip_h = owner_->current_task_.height_;
+
+    std::ostringstream command;
+    command << "mencoder mf://" << directory.string() << "/*.jpg -quiet -mf w=" << clip_w << ":h=" << clip_h << ":fps=" << fps << ":type=jpg -ovc lavc -lavcopts vcodec=mjpeg -oac copy -of lavf -lavfopts format=mov -o " << output_movie.string();
+    std::cout << "Lauching $ " << command.str() << std::endl;  
+    bool ret_val = subprocess::run_command(command.str()); // blocking call
     //std::cout << "Done with $ " << command << std::endl;
     if (ret_val != 0)
         std::cout << "Mencoder's return value was " << ret_val << std::endl;
     else
         std::cout << "Success!" << std::endl;
     // rename movie file
-    std::string final_movie = owner_->get_result_directory() + "/movie-" + datetime_started  + ".mov"; 
     try 
     {
-        fs::copy_file(fs::path(output_movie), fs::path(final_movie));
+        fs::copy_file(fs::path(output_movie), fs::path(final_file_name_));
         // the old file will be deleted with the whole dir
     }
     catch(fs::filesystem_error e) 
@@ -136,7 +145,7 @@ void SaverWorker::operator()()
         success_ = false;
         return;
     }
-    std::cout << "Done creating movie " << final_movie << std::endl;
+    std::cout << "Done creating movie " << final_file_name_ << std::endl;
     success_ = true;
     return;
 }
