@@ -1,9 +1,8 @@
 /*
  * Toonloop
  *
- * Copyright 2010 Alexandre Quessy
- * <alexandre@quessy.net>
- * http://www.toonloop.com
+ * Copyright (c) 2010 Alexandre Quessy <alexandre@quessy.net>
+ * Copyright (c) 2010 Tristan Matthews <le.businessman@gmail.com>
  *
  * Toonloop is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,10 +30,13 @@ class SaverWorker
 {
     public:
         SaverWorker(MovieSaver *owner);
+        void set_final_options(const std::string &datetime_started, const std::string &final_file_name);
         void operator()();
         bool success_;
     private:
         MovieSaver *owner_;
+        std::string datetime_started_;
+        std::string final_file_name_;
 };
 
 #endif
